# encoding: utf-8

# author: BrikerMan
# contact: eliyar917@gmail.com
# blog: https://eliyar.biz

# file: __init__.py
# time: 2:30 下午

from .gru_encoder import GRUEncoder

if __name__ == "__main__":
    pass
