# encoding: utf-8

# author: BrikerMan
# contact: eliyar917@gmail.com
# blog: https://eliyar.biz

# file: __init__.py
# time: 10:44 下午

from kashgari.metrics.multi_label_classification import multi_label_classification_report
from kashgari.metrics.sequence_labeling import sequence_labeling_report

if __name__ == "__main__":
    pass
